REM -------------------------------------------------------------------------
REM
REM                         P A R T Y
REM
REM               A social interaction simulation based
REM               on the ideal distance between people.
REM
REM
REM  A group of people with varying preferences towards each other
REM  move around at a party. These preferences define the ideal distance
REM  between them and the others, ranging from affectionate closeness, or
REM  aloof or even wary distance, to complete indifference.
REM
REM  Occasionally, people will get hungry or thirsty and go to the table
REM  in the south west, or may suddenly have the urge to go to the restroom
REM  in the north east. This will cause shifts in other people`s positions.
REM
REM  People on their way to the bathroom will be ignored by the rest until
REM  they come out, but not so on their way to the table.
REM
REM  Preferences can be altered during the game by using the cursor arrows
REM  to select the value to be altered and then press + or -.
REM
REM  Each person compares the real and ideal distances to all the others,
REM  deriving an overall satisfaction index for their current position, and
REM  also for the 4 surrounding positions - north, south, east and west. A
REM  move to one of these will be made if the satisfaction index is better.
REM  A position may be occuppied by more than one person at any given time.
REM
REM  Preferences can be saved or loaded pressing "S" or "L". Remember the
REM  name! You can give it any name, with or without extension.
REM  Files are stored in the same directory as the programme is in.
REM  CAPS LOCK must be OFF.
REM
REM  Legal Stuff
REM  ------------
REM  This programme is freeware, produced solely for the enjoyment of the
REM  hobby programmer's community. No responsibility is taken for any loss
REM  or damage incurred directly or indirectly from its use. The original
REM  unaltered files may be freely distributed, uploaded, downloaded and
REM  shared. Please maintain proper credit, of course, and make no financial
REM  gain whatsoever.
REM
REM -------------------------------------------------------------------------
REM                    Aleatorylamp - May 2012
REM -------------------------------------------------------------------------

CLS
RANDOMIZE TIMER
SCREEN 12
WIDTH 80, 60
REM ---------- Print out blue floor.
floorwidth = 24
floorheight = 24

COLOR 1
FOR v = 1 TO floorheight
    FOR h = 1 TO floorwidth
        LOCATE v, h
        PRINT CHR$(176)
    NEXT h
NEXT v

DIM P$(14)

P$(1) = "King"
P$(2) = "Queen"
P$(3) = "Horseman"
P$(4) = "Maid"
P$(5) = "Bishop"
P$(6) = "Witch"
P$(7) = "Footman"
P$(8) = "Nurse"
P$(9) = "Count"
P$(10) = "Duchess"
P$(11) = "Gardener"
P$(12) = "Princess"
P$(13) = "Table"
P$(14) = "Restroom"

REM --------- List names to the right of floor
REM
FOR n = 2 TO 24 STEP 2
    LOCATE n, floorwidth + 2
    COLOR 2 + n / 2
    PRINT P$(n / 2)
NEXT n
COLOR 9
LOCATE 29, 1: COLOR 9: PRINT "Cursor Arrows:";: COLOR 2: PRINT " Select"
LOCATE 30, 1: COLOR 9: PRINT "      + or - :";: COLOR 14: PRINT " Alter"
LOCATE 31, 1: COLOR 9: PRINT "           S :";: COLOR 3: PRINT " Save"
LOCATE 32, 1: COLOR 9: PRINT "           L :";: COLOR 12: PRINT " Load"
LOCATE 33, 1: COLOR 9: PRINT "           Q :";: COLOR 4: PRINT " Quit"
LOCATE 35, 1: COLOR 8: PRINT "  (CAPS LOCK OFF)"

LOCATE 46, 32: COLOR 6: PRINT "Higher values = keep more distance"
LOCATE 47, 32: PRINT "Lower values  = closer approach"
LOCATE 48, 32: PRINT "         Zero = indifference"

REM --------- Ideal distances of each character to the
REM --------- others, to the table and the restroom.

DIM pref(12, 14)

LOCATE 27, floorwidth + 5: COLOR 3: PRINT "* * *  ";
COLOR 12: PRINT "   Ideal Distance Table   ";
COLOR 3: PRINT "  * * *"

REM --------- Print initials along top row and first col of prefs table

FOR n = 1 TO 14
    COLOR 7
    IF n = 13 THEN COLOR 6
    IF n = 14 THEN COLOR 9
    LOCATE 29, floorwidth + 2 + n * 3
    PRINT LEFT$(P$(n), 1)
    IF n < 13 THEN
        COLOR 2 + n
        LOCATE 30 + n, floorwidth + 2
        PRINT LEFT$(P$(n), 1)
    END IF
NEXT n

REM --------- Read and print out preference table

RESTORE preferences
FOR n = 1 TO 12
    FOR P = 1 TO 14
        READ A
        pref(n, P) = A
        LOCATE n + 30, floorwidth + 1 + P * 3
        COLOR 2
        IF P = 13 THEN COLOR 6
        IF P = 14 THEN COLOR 9
        IF P = n THEN COLOR 4
        PRINT pref(n, P)
    NEXT P
NEXT n

REM ---------- Higher values indicate a greater ideal
REM ---------- distance and 0 means total indifference.

preferences:
REM  k q j m b  w  f n c d g p t  r

DATA 0,3,2,1,4,10,4,10,1,10,2,7,0,0
DATA 5,0,1,3,7,10,6,3,1,8,7,3,0,0
DATA 3,1,0,1,8,10,3,1,5,1,4,1,0,0

DATA 1,2,10,0,10,20,3,7,1,4,2,5,0,0
DATA 5,5,5,1,0,10,5,1,7,1,7,7,0,0
DATA 1,1,1,1,10,0,1,1,1,1,1,1,0,0

DATA 3,3,3,1,9,10,0,1,7,3,3,1,0,0
DATA 3,3,3,1,9,10,3,0,1,7,3,3,0,0
DATA 3,3,3,1,3,10,7,7,0,3,3,7,0,0

DATA 3,3,3,1,3,10,7,7,7,0,3,3,0,0
DATA 3,3,3,1,3,10,7,3,7,3,0,7,0,0
DATA 3,3,10,3,10,10,7,3,7,8,1,0,0,0

REM ---------- Matrix for 12 chatacters, table and restroom
REM ---------- Pos 1 and 2 are v and h positions.
REM ---------- Pos 3 is insatisfaction index of current square,
REM ---------- and Pos 4, 5, 6, and 7 that of square above,
REM ---------- below, right and left respectively.

DIM P(14, 10)

REM ---------- Print characters' positions on floor

RANDOMIZE TIMER
FOR n = 1 TO 12
    P(n, 1) = 2 + INT(RND * (floorheight - 2))
    P(n, 2) = 2 + INT(RND * (floorwidth - 2))
    IF n < 13 THEN COLOR 2 + n
    IF n = 13 THEN COLOR 6
    IF n = 14 THEN COLOR 9
    LOCATE P(n, 1), P(n, 2)
    PRINT LEFT$(P$(n), 1)
NEXT n
REM ---------- Position table at bottom left
P(13, 1) = floorheight - 2
P(13, 2) = 3
                         
LOCATE P(13, 1), P(13, 2)
COLOR 6
PRINT LEFT$(P$(13), 1)

REM ---------- Position restroom top right corner
P(14, 1) = 3
P(14, 2) = floorwidth - 2
LOCATE P(14, 1), P(14, 2)
COLOR 9
PRINT LEFT$(P$(14), 1)

n = 1
pe = 1
COLOR 14

REM -------------------------Main loop ---------------------------------

REM ---------------Modify character preferences within game-------------
DO

    DO
        k$ = INKEY$
        IF k$ = "q" THEN GOTO hhh
        COLOR 14
        LOCATE n + 30, floorwidth + 1 + pe * 3
        PRINT pref(n, pe)
        LOCATE 44, floorwidth + 5: COLOR 2 + n: PRINT P$(n);
        IF n <> pe THEN
            PRINT "'s preference to ";
            IF pe < 13 THEN COLOR pe + 2
            IF pe = 13 THEN COLOR 6
            IF pe = 14 THEN COLOR 9
            PRINT P$(pe); "    "
        ELSE PRINT "                           "
        END IF

        REM ---------- Each character is processed: First sent to position
        REM ---------- analysis routine and then to move routine.

        FOR ch = 1 TO 12
            REM --------- I have to go to the loo!
            IF INT(RND * 500) < 1 THEN
                IF pref(ch, 13) = 0 THEN
                    pref(ch, 14) = 1
                    LOCATE ch + 30, floorwidth + 1 + 14 * 3
                    COLOR 9
                    PRINT pref(ch, 14)
                END IF
            END IF

            REM --------- I`m hungry, and I want a drink!
            IF INT(RND * 500) < 1 THEN
                IF pref(ch, 14) = 0 THEN
                    pref(ch, 13) = 1
                    LOCATE ch + 30, floorwidth + 1 + 13 * 3
                    COLOR 6
                    PRINT pref(ch, 13)
                END IF
            END IF


            GOSUB analysis
            GOSUB move
            SOUND 37, .1
            _DELAY .036
            REM ----------table position
            LOCATE P(13, 1), P(13, 2)
            COLOR 6
            PRINT LEFT$(P$(13), 1)

            REM ----------restroom position
            LOCATE P(14, 1), P(14, 2)
            COLOR 9
            PRINT LEFT$(P$(14), 1)

        NEXT ch

    LOOP WHILE k$ = ""

    k = ASC(RIGHT$(k$, 1))
    SOUND 300, .3
    po = pe
    no = n
    COLOR 2
    IF pe = 13 THEN COLOR 6
    IF pe = 14 THEN COLOR 9
    IF pe = n THEN COLOR 4
    LOCATE no + 30, floorwidth + 1 + po * 3
    PRINT pref(no, po)

    SELECT CASE k

        CASE 13
            END

        CASE 80
            n = n + 1
            IF n > 12 THEN n = 1

        CASE 72
            n = n - 1
            IF n < 1 THEN n = 12

        CASE 77
            pe = pe + 1
            IF pe > 14 THEN pe = 1

        CASE 75
            pe = pe - 1
            IF pe < 1 THEN pe = 14

        CASE 43
            IF pe = n THEN GOTO hui
            IF pe = 13 THEN IF pref(n, 14) = 1 THEN GOTO hui
            IF pe = 14 THEN IF pref(n, 13) = 1 THEN GOTO hui
            pref(n, pe) = pref(n, pe) + 1
            IF pe > 12 THEN IF pref(n, pe) > 1 THEN pref(n, pe) = 1
            IF pref(n, pe) > 30 THEN pref(n, pe) = 0
            hui:

        CASE 45
            IF pe = n THEN GOTO huh
            pref(n, pe) = pref(n, pe) - 1
            IF pe > 12 THEN IF pref(n, pe) < 0 THEN pref(n, pe) = 0
            IF pref(n, pe) < 0 THEN pref(n, pe) = 30
            huh:

        CASE 115: REM -------- SAVE

            REM ------------- opens channel 1 to WRITE file$ nombrepref$
            conten$ = ""
            LOCATE 39, 1: COLOR 12: PRINT "Remember the name!"
            LOCATE 40, 1: COLOR 2: PRINT "No entry: Return"
            LOCATE 42, 1: COLOR 6
            INPUT "Save name"; nombrepref$
            IF nombrepref$ = "" THEN GOTO kjl
            OPEN nombrepref$ FOR OUTPUT AS #1
            FOR sn = 1 TO 12
                FOR sm = 1 TO 14
                    conten$ = conten$ + CHR$(pref(sn, sm))
                NEXT sm
            NEXT sn
            WRITE #1, conten$
            CLOSE #1

            FOR sn = 1 TO 12
                FOR sm = 1 TO 14
                    LOCATE sn + 30, floorwidth + 1 + sm * 3
                    COLOR 2
                    IF sm = 13 THEN COLOR 6
                    IF sm = 14 THEN COLOR 9
                    IF sm = sn THEN COLOR 4
                    PRINT pref(sn, sm)
                NEXT sm
            NEXT sn

            kjl:
            LOCATE 39, 1: PRINT "                   "
            LOCATE 40, 1: PRINT "                   "
            LOCATE 42, 1: PRINT "                   "



        CASE 108: REM -------- LOAD

            REM ------------ opens channel 1 to READ file$ nombrepref$
            LOCATE 39, 1: COLOR 14: PRINT "Remember the name!"
            LOCATE 40, 1: COLOR 2: PRINT "No entry: Return"
            LOCATE 42, 1: COLOR 9
            INPUT "Load name"; nombrepref$
            IF nombrepref$ = "" THEN GOTO jkl
            OPEN nombrepref$ FOR INPUT AS #1
    
            REM eliminate first character
            Character$ = INPUT$(1, # 1)

            PRINT characters$
            FOR ln = 1 TO 12
                FOR lm = 1 TO 14
                    Character$ = INPUT$(1, # 1)
                    pref(ln, lm) = ASC(Character$)
                NEXT lm
            NEXT ln
            CLOSE #1

            FOR ln = 1 TO 12
                FOR lm = 1 TO 14
                    LOCATE ln + 30, floorwidth + 1 + lm * 3
                    COLOR 2
                    IF lm = 13 THEN COLOR 6
                    IF lm = 14 THEN COLOR 9
                    IF lm = ln THEN COLOR 4
                    PRINT pref(ln, lm)
                NEXT lm
            NEXT ln
            jkl:
            LOCATE 39, 1: PRINT "                   "
            LOCATE 40, 1: PRINT "                   "
            LOCATE 42, 1: PRINT "                   "

    END SELECT

LOOP

hhh:
                                                             
END

analysis:
REM ---------- Character analyses distance to the 11 other characters,
REM ---------- to the table and the restroom, from current position and
REM ---------- from squares above, below, right and left.

sumdif = 0
sumdif1 = 0
sumdif2 = 0
sumdif3 = 0
sumdif4 = 0

FOR s = 1 TO 14

    LOCATE floorheight + 1, 1: COLOR 9: PRINT "Analysing ";
    COLOR ch + 2: PRINT P$(ch); "    ";
    IF s < 13 THEN COLOR s + 2
    IF s = 13 THEN COLOR 6
    IF s = 14 THEN COLOR 9

    REM ---------- but not if it's the distance to him/herself!
    REM IF s = ch THEN GOTO pio

    REM ---------- or ignore the other character if that preference is 0!
    IF pref(ch, s) = 0 THEN GOTO pio

    REM ---------- or ignore the character if on the way to the restroom
    REM ---------- ...but not coming back from the restroom!
    IF s < 13 THEN
        IF pref(s, 14) = 1 THEN GOTO pio
    END IF

    vreldist = ABS(P(ch, 1) - P(s, 1))
    vreldist1 = ABS(P(ch, 1) - 1 - P(s, 1))
    vreldist2 = ABS(P(ch, 1) + 1 - P(s, 1))
    hreldist = ABS(P(ch, 2) - P(s, 2))
    hreldist1 = ABS(P(ch, 2) + 1 - P(s, 2))
    hreldist2 = ABS(P(ch, 2) - 1 - P(s, 2))

    REM ---------- Pythagoras to get distances

    vreldist = vreldist * vreldist
    hreldist = hreldist * hreldist
    vreldist1 = vreldist1 * vreldist1
    hreldist1 = hreldist1 * hreldist1
    vreldist2 = vreldist2 * vreldist2
    hreldist2 = hreldist2 * hreldist2

    reldist = SQR(vreldist + hreldist)
    reldist1 = SQR(vreldist1 + hreldist)
    reldist2 = SQR(vreldist2 + hreldist)
    reldist3 = SQR(vreldist + hreldist1)
    reldist4 = SQR(vreldist + hreldist2)


    REM ---------- Compare real distances to other characters with
    REM ---------- preferred ones. Larger differences = lower satisfaction.

    difdist = ABS(pref(ch, s) - reldist)
    difdist1 = ABS(pref(ch, s) - reldist1)
    difdist2 = ABS(pref(ch, s) - reldist2)
    difdist3 = ABS(pref(ch, s) - reldist3)
    difdist4 = ABS(pref(ch, s) - reldist4)


    REM ---------- Sum of differences for current and other 4 positions.

    IF s < 13 THEN
        sumdif = sumdif + difdist
        sumdif1 = sumdif1 + difdist1
        sumdif2 = sumdif2 + difdist2
        sumdif3 = sumdif3 + difdist3
        sumdif4 = sumdif4 + difdist4
    END IF

    REM ---------- Table override factor
    IF s = 13 THEN
        IF pref(ch, s) = 1 THEN
            sumdif = difdist
            sumdif1 = difdist1
            sumdif2 = difdist2
            sumdif3 = difdist3
            sumdif4 = difdist4
            IF difdist = 0 THEN
                pref(ch, s) = 0
                LOCATE ch + 30, floorwidth + 1 + s * 3
                PRINT pref(ch, s)
            END IF

        END IF
    END IF

    REM ---------- Restroom override factor
    IF s = 14 THEN
        IF pref(ch, s) = 1 THEN
            sumdif = difdist
            sumdif1 = difdist1
            sumdif2 = difdist2
            sumdif3 = difdist3
            sumdif4 = difdist4
            IF difdist = 0 THEN
                pref(ch, s) = 0
                LOCATE ch + 30, floorwidth + 1 + s * 3
                PRINT pref(ch, s)
            END IF
        END IF
    END IF

    REM ---------- and is stored as of the 3rd string position.
    P(ch, 3) = sumdif
    P(ch, 4) = sumdif1
    P(ch, 5) = sumdif2
    P(ch, 6) = sumdif3
    P(ch, 7) = sumdif4

    pio:
NEXT s

REM ---------- The square with the lowest value is chosen.
choice = 3
FOR tim = 1 TO 5
    FOR pr = 4 TO 7
        IF P(ch, choice) > P(ch, pr) THEN choice = pr
    NEXT pr
NEXT tim

RETURN

move:
M = ch
IF M = 13 THEN COLOR 6: GOTO sio
IF M = 14 THEN COLOR 9: GOTO sio

COLOR 1
LOCATE P(M, 1), P(M, 2)
PRINT CHR$(176)

IF choice = 4 THEN P(M, 1) = P(M, 1) - 1
IF choice = 5 THEN P(M, 1) = P(M, 1) + 1
IF choice = 6 THEN P(M, 2) = P(M, 2) + 1
IF choice = 7 THEN P(M, 2) = P(M, 2) - 1

IF P(M, 1) < 2 THEN P(M, 1) = 2
IF P(M, 1) > floorheight - 2 THEN P(M, 1) = floorheight - 2
IF P(M, 2) < 2 THEN P(M, 2) = 2
IF P(M, 2) > floorwidth - 2 THEN P(M, 2) = floorwidth - 2

COLOR M + 2

sio:
REM ---------- New position for character
LOCATE P(M, 1), P(M, 2)
PRINT LEFT$(P$(M), 1)

RETURN



